#pragma once

#define STB_IMAGE_WRITE_IMPLEMENTATION

#include <vector>
#include <string>
#include <iostream>

#include "stb_image_write.hpp"
#include "Renderer.hpp"

namespace massivedynamic {

typedef uint32_t Color;

class PNGRenderer : public Renderer {
	private:
		constexpr static Color BLACK = 0xFF000000;
		constexpr static Color WHITE = 0xFFFFFFFF;

		std::vector<Color> bitmap;

		void drawPixelScaled(size_t x, size_t y, Color color) {
			size_t pixelSize = this->targetSize / (this->sourceSize + 2);

			if (pixelSize < 1) {
				std::cerr << "ERROR: output file size is too small" << std::endl;
				exit(1);
			}

			size_t absoluteX = pixelSize * x;
			size_t absoluteY = pixelSize * y;

			for (size_t localY = absoluteY + pixelSize; localY < absoluteY + pixelSize + pixelSize; localY++) {
				for (size_t localX = absoluteX + pixelSize; localX < absoluteX + pixelSize + pixelSize; localX++) {
					size_t absPos = localY * this->targetSize + localX;
					this->bitmap.at(absPos) = color;
				}
			}
		}
	public:
		PNGRenderer(const std::vector<bool>& pixels, size_t sourceSize, size_t targetSize) : Renderer(pixels, sourceSize, targetSize) {
			if (this->targetSize == 0)
				this->targetSize = sourceSize;
			this->bitmap = std::vector<Color>(targetSize * targetSize, 0xFFFFFFFF);
		}

		virtual void render(const std::string& filename) override {
			for (size_t y = 0; y < sourceSize; y++) {
				for (size_t x = 0; x < sourceSize; x++) {
					Color color = pixels.at(y * sourceSize + x) ? PNGRenderer::BLACK : PNGRenderer::WHITE;
					this->drawPixelScaled(x, y, color);
				}
			}

			stbi_write_png(filename.c_str(), targetSize, targetSize, 4, this->bitmap.data(), targetSize * sizeof(Color));
		}

};

}