
#include <iostream>
#include <vector>
#include "qrcodegen.hpp"
#include "QR.hpp"
#include "ConsoleRenderer.hpp"
#include "PNGRenderer.hpp"
#include "SVGRenderer.hpp"

namespace massivedynamic {

QR::QR(const std::string& data, std::string outputFile, size_t size, Type type, Format format) : outputFile(std::move(outputFile)) {

	qrcodegen::QrCode::Ecc errorCorrectionLevel =  qrcodegen::QrCode::Ecc::HIGH;

	switch(type) {
		case Type::SMALL:
		 	errorCorrectionLevel = qrcodegen::QrCode::Ecc::LOW;
			break;
		case Type::MEDIUM:
			errorCorrectionLevel = qrcodegen::QrCode::Ecc::MEDIUM;
			break;
		default:
		case Type::LARGE:
			errorCorrectionLevel = qrcodegen::QrCode::Ecc::HIGH;
			break;
	}

	qrcodegen::QrCode qr = qrcodegen::QrCode::encodeText(data.c_str(), errorCorrectionLevel);
	
	this->renderers.insert({Format::CONSOLE, std::make_unique<ConsoleRenderer>(this->pixels, qr.getSize(), size)});
	this->renderers.insert({Format::PNG, std::make_unique<PNGRenderer>(this->pixels, qr.getSize(), size)});
	this->renderers.insert({Format::SVG, std::make_unique<SVGRenderer>(this->pixels, qr.getSize(), size)});

	for (int y = 0; y < qr.getSize(); y++) {
		for (int x = 0; x < qr.getSize(); x++) {
			bool isFilled = qr.getModule(x, y);
			this->pixels.push_back(isFilled ? true : false);
		}
	}

	this->renderers.at(format)->render(this->outputFile);
}

}