#pragma once

#include <sstream>
#include <fstream>

#include "Renderer.hpp"

namespace massivedynamic {

	class SVGRenderer : public Renderer {
		public:
			SVGRenderer(const std::vector<bool>& pixels, size_t sourceSize, size_t targetSize) : Renderer(pixels, sourceSize, targetSize) {}
			virtual void render(const std::string& filename) override {
				std::stringstream file;

				std::string base = R"EOF(<?xml version="1.0" encoding="utf-8"?>
<svg version="1.1" id="qrcode" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
				)EOF";

				file << base;
				file << "viewBox=\"0 0 " << this->targetSize << " " << this->targetSize << "\"  xml:space=\"preserve\">" << '\n';
				file << "    <rect x=\"0\" y=\"0\" width=\"" << this->targetSize << "\" height=\"" << this->targetSize << "\" style=\"fill: white;\" />" << '\n';
				size_t pixelSize = this->targetSize / (this->sourceSize + 2);	

				for (size_t y = 0; y < this->sourceSize; y++) {
					for (size_t x = 0; x < this->sourceSize; x++) {
						if (!this->pixels.at(y * this->sourceSize + x))
							continue;
						size_t rectX = x * pixelSize + pixelSize;
						size_t rectY = y * pixelSize + pixelSize;
						file << "    <rect x=\"" << rectX << "\" y=\"" << rectY << "\" width=\"" << pixelSize << "\" height=\"" << pixelSize << "\" style=\"fill: black; stroke: black; stroke-width: 3;\" />" << '\n';
					}
				}

				file << "</svg>" << std::endl;

				std::ofstream outFile;
				outFile.open(filename);
				outFile << file.str();
				outFile.close();
			}

	};

}